local allowed = nil
local warned = {}
local lastAllowedIncomingPlayer = nil
local disabled = nil

local function ShouldShow(name)
	return allowed[name] or rpfilDB[name]
end

function rpfil_Disable() disabled = true end
function rpfil_Enable() disabled = nil end

local function oocplayerfilter()
	if not allowed or (type(arg6) == "string" and arg6:find("GM")) or disabled then return end
	local player = arg2
	local text = arg1
	if not ShouldShow(player) then
		if lastAllowedIncomingPlayer then
			ChatEdit_SetLastTellTarget(lastAllowedIncomingPlayer)
		end
		return true
	else
		lastAllowedIncomingPlayer = player
	end
end

local function outgoingFilter()
	if not allowed or (type(arg6) == "string" and arg6:find("GM")) or disabled then return end
	local player = arg2
	local text = arg1
	if not allowed[player] then
		rpfilDB[player] = true
	end
end

ChatFrame_AddMessageEventFilter("CHAT_MSG_CHANNEL", oocplayerfilter)
ChatFrame_AddMessageEventFilter("CHAT_MSG_SAY", oocplayerfilter)
ChatFrame_AddMessageEventFilter("CHAT_MSG_YELL", oocplayerfilter)
ChatFrame_AddMessageEventFilter("CHAT_MSG_EMOTE", oocplayerfilter)
ChatFrame_AddMessageEventFilter("CHAT_MSG_WHISPER", oocplayerfilter)
ChatFrame_AddMessageEventFilter("CHAT_MSG_WHISPER_INFORM", outgoingFilter)

local function InsertFriends()
	if not allowed then allowed = {} end
	-- Find all friends
	local numFriends = GetNumFriends()
	for i = 1, numFriends do
		local name = GetFriendInfo(i)
		if name then
			allowed[name] = true
		end
	end
end

local function InsertGuildMembers()
	if not allowed then allowed = {} end
	-- Find all guild members.
	local numGuildMembers = GetNumGuildMembers()
	for i = 1, numGuildMembers do
		local name = GetGuildRosterInfo(i)
		if name then
			allowed[name] = true
		end
	end
end

local frame = CreateFrame("Frame")
frame:SetScript("OnEvent", function()
	if event == "PLAYER_LOGIN" then
		if type(rpfilDB) ~= "table" then rpfilDB = {} end

		if IsInGuild() then GuildRoster() end
		ShowFriends()
	elseif event == "PARTY_MEMBERS_CHANGED" then
		if not allowed then allowed = {} end
		for i = 1, GetNumPartyMembers() do
			local n = UnitName("party"..i)
			if n then allowed[n] = true end
		end
	elseif event == "RAID_ROSTER_UPDATE" then
		if not allowed then allowed = {} end
		for i = 1, GetNumRaidMembers() do
			local n = GetRaidRosterInfo(i)
			if n then allowed[n] = true end
		end
	elseif event == "FRIENDLIST_UPDATE" then
		InsertFriends()
	elseif event == "GUILD_ROSTER_UPDATE" then
		InsertGuildMembers()
	end
end)

frame:RegisterEvent("GUILD_ROSTER_UPDATE")
frame:RegisterEvent("FRIENDLIST_UPDATE")
frame:RegisterEvent("PLAYER_LOGIN")
frame:RegisterEvent("PARTY_MEMBERS_CHANGED")
frame:RegisterEvent("RAID_ROSTER_UPDATE")
